@@ -13,7 +13,7 @@ from lxml.etree import XMLParser, parse, fromstring, Comment, HTMLParser
 from xmldt.element import Element, toxml
 import sys
 
-__version__ = "0.0.5"
+__version__ = "0.0.6"
 __docformat__ = 'markdown'
 
 
@@ -31,6 +31,7 @@ class XmlDt:
 
     def __init__(self, strip=False, empty=False, html=False):
         self._parser = XMLParser(recover=True) if not html else HTMLParser(recover=True)
+        self._chaining = None
         self._path = []
         self._hooks = []
         self._strip = strip
@@ -97,19 +98,18 @@ class XmlDt:
             # add to path only if required
             self._path.append(Element(element.tag, {**element.attrib}, None, self))
 
-            tag_handler = None
+            tag_handlers = []
             for hook in self._hooks:
-                tag_handler = hook(self._path[-1])
-                if tag_handler is not None:
-                    break
+                tag_handlers += hook(self._path[-1])
 
-            if tag_handler is None:
-                if element.tag in self._alias:
-                    tag_handler = self._alias[element.tag]
-                else:
-                    tag_handler = getattr(self, element.tag, self.__default__)
-                    if not callable(tag_handler):
-                        tag_handler = self.__default__
+            if element.tag in self._alias:
+                tag_handlers.append(self._alias[element.tag])
+            else:
+                handler = getattr(self, element.tag, None)
+                if handler is not None and callable(handler):
+                    tag_handlers.append(handler)
+
+            tag_handlers.append(self.__default__)
 
             # given an element, process children, returning a list of pairs
             contents = self._dt(element)      # not zero:
@@ -137,7 +137,17 @@ class XmlDt:
             else:
                 raise Exception(f"Element type not recognized: {tag_type}")
 
-            return tag_handler(elem)
+            result = None  # should never happen, default is always there
+            for handler in tag_handlers:
+                self._chaining = None
+                result = handler(elem)
+                if self._chaining is not None:
+                    elem = self._chaining
+                    self._chain = False
+                else:
+                    break
+
+            return result
 
     def _dt(self, tree):
         children = []    # agora retorna lista de pares (tag, conte)*
@@ -163,13 +173,19 @@ class XmlDt:
             "father": self._father,
             "gfather": self._gfather,
             "root": self._root,
-            "in_context": lambda: lambda x: self._in_context(x)  # lovely!
+            "hooks": lambda: self._hooks,
+            "add_hook": lambda: lambda x: self._hooks.insert(0, x),
+            "in_context": lambda: lambda x: self._in_context(x),  # lovely!
+            "chain": lambda: lambda elem: self._chain(elem)
         }
         if item in dt:
             return dt[item]()
         else:
             return None
 
+    def _chain(self, elem):
+        self._chaining = elem
+
     def _in_context(self, tag):
         return len([e for e in self._path if e.tag == tag]) > 0
 
