@@ -21,7 +21,7 @@ class HtmlDt (XmlDt):
             method.has_class: method for method in {
                 getattr(self, name) for name in dir(self)
                 if callable(getattr(self, name)) and hasattr(getattr(self, name), "has_class")}}
-        self._hooks.append(lambda x: self._class_hook(x))
+        self["add_hook"](lambda x: self._class_hook(x))
 
     def __call__(self, *args, **kwargs):
         super().__call__(*args, **kwargs)
@@ -30,11 +30,12 @@ class HtmlDt (XmlDt):
         return super().__new__(cls, *args, **{**kwargs, "html": True})
 
     def _class_hook(self, element):
+        handlers = []
         if "class" in element:
-            classes = re.split(r'\s+', element["class"])
-            for c in classes:
+            element["class"] = re.split(r'\s+', element["class"])
+            for c in element["class"]:
                 if c in self._classes:
-                    return self._classes[c]
-        return None
+                    handlers.append(self._classes[c])
+        return handlers
 
 
